"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initMiddleware = exports.METLO_META = void 0;
const constants_1 = require("../utils/constants");
const rust_binds_1 = require("../rust_binds");
const logger_1 = require("../utils/logger");
exports.METLO_META = {
    key: "",
    host: "",
    backendPort: -1,
    collectorPort: -1,
    opts: {},
    maxBodyCaptureSize: 5 * 1024,
};
const initMiddleware = (props) => {
    try {
        new URL(props.host);
    }
    catch (err) {
        (0, logger_1.Logger)().log("error", err);
        throw new Error(`Couldn't load metlo. Host is not a proper url : ${props.host}`);
    }
    exports.METLO_META.host = props.host;
    exports.METLO_META.key = props.key;
    exports.METLO_META.backendPort = props.backendPort
        ? props.backendPort
        : props.host.includes("app.metlo.com")
            ? 443
            : 8000;
    exports.METLO_META.collectorPort = props.collectorPort ? props.collectorPort : 8081;
    exports.METLO_META.encryptionKey = props.encryptionKey;
    exports.METLO_META.logLevel = props.logLevel;
    (0, logger_1.Logger)(props.logLevel);
    exports.METLO_META.getUser = props.getUser;
    exports.METLO_META.rejectResponse = props.rejectResponse;
    if (props.maxBodyCaptureSize) {
        exports.METLO_META.maxBodyCaptureSize = props.maxBodyCaptureSize;
    }
    else {
        exports.METLO_META.maxBodyCaptureSize = constants_1.MAX_BODY_SIZE;
    }
    let disableBlocking = false;
    if (props.disableBlocking != undefined) {
        disableBlocking = Boolean(props.disableBlocking);
    }
    else {
        let flagValue = process.env.METLO_DISABLE_BLOCKING;
        if (flagValue !== undefined &&
            (flagValue === "true" || flagValue === "1" || flagValue === "yes")) {
            disableBlocking = true;
        }
    }
    if (disableBlocking) {
        (0, logger_1.Logger)().log("debug", "Request Blocking by Metlo has been disabled");
    }
    exports.METLO_META.disableBlocking = disableBlocking;
    (0, rust_binds_1.startMetlo)(exports.METLO_META.host, exports.METLO_META.key, exports.METLO_META.backendPort, exports.METLO_META.collectorPort, exports.METLO_META.encryptionKey, exports.METLO_META.logLevel)
        .then(() => { })
        .catch(err => {
        (0, logger_1.Logger)().log("error", "Metlo encountered an error while initializing ");
        (0, logger_1.Logger)().log("error", err);
    });
};
exports.initMiddleware = initMiddleware;
exports.default = exports.initMiddleware;
//# sourceMappingURL=index.js.map