import { ConfigOptions } from "./types"
import { initMiddleware } from "./middlewares"

import ExpressModule, {
  MetloConfigExpress,
  dummyModule as ExpressDummyMiddleware,
} from "./middlewares/express"
import KoaModule, {
  MetloConfigKoa,
  dummyModule as KoaDummyMiddleware,
} from "./middlewares/koa"
import FastifyModule, {
  MetloConfigFastify,
  dummyModule as FastifyDummyMiddleware,
  fastifyRequest,
  fastifyResponse,
} from "./middlewares/fastify"
import { METLO_ENABLED } from "./utils"

import {
  Request as ExpressRequest,
  Response as ExpressResponse,
} from "express-serve-static-core"
import Application from "koa"

export const initFastify = <
  req extends fastifyRequest,
  res extends fastifyResponse,
>(
  params: MetloConfigFastify<req, res>,
) => {
  if (METLO_ENABLED()) {
    initMiddleware(params)
    return FastifyModule()
  } else {
    FastifyDummyMiddleware()
  }
}

export const initExpress = <
  req extends ExpressRequest,
  res extends ExpressResponse,
>(
  params: MetloConfigExpress,
) => {
  if (METLO_ENABLED()) {
    initMiddleware(params)
    return ExpressModule<req, res>()
  } else {
    ExpressDummyMiddleware()
  }
}

export const initKoa = <ctx extends Application.ExtendableContext>(
  params: MetloConfigKoa<ctx>,
) => {
  if (METLO_ENABLED()) {
    initMiddleware(params)
    return KoaModule<ctx>()
  }
  return KoaDummyMiddleware()
}
