import { MAX_BODY_SIZE } from "../utils/constants"
import { startMetlo } from "../rust_binds"
import { MetloConfig } from "../types"
import { Logger } from "../utils/logger"

export const METLO_META: MetloConfig = {
  key: "",
  host: "",
  backendPort: -1,
  collectorPort: -1,
  opts: {},
  maxBodyCaptureSize: 5 * 1024,
}

export const initMiddleware = (props: MetloConfig) => {
  try {
    new URL(props.host)
  } catch (err) {
    Logger().log("error", err)
    throw new Error(
      `Couldn't load metlo. Host is not a proper url : ${props.host}`,
    )
  }
  METLO_META.host = props.host

  METLO_META.key = props.key

  METLO_META.backendPort = props.backendPort
    ? props.backendPort
    : props.host.includes("app.metlo.com")
    ? 443
    : 8000

  METLO_META.collectorPort = props.collectorPort ? props.collectorPort : 8081

  METLO_META.encryptionKey = props.encryptionKey

  METLO_META.logLevel = props.logLevel
  Logger(props.logLevel)

  METLO_META.getUser = props.getUser

  METLO_META.rejectResponse = props.rejectResponse

  if (props.maxBodyCaptureSize) {
    METLO_META.maxBodyCaptureSize = props.maxBodyCaptureSize
  } else {
    METLO_META.maxBodyCaptureSize = MAX_BODY_SIZE
  }

  let disableBlocking = false
  if (props.disableBlocking != undefined) {
    disableBlocking = Boolean(props.disableBlocking)
  } else {
    let flagValue = process.env.METLO_DISABLE_BLOCKING
    if (
      flagValue !== undefined &&
      (flagValue === "true" || flagValue === "1" || flagValue === "yes")
    ) {
      disableBlocking = true
    }
  }
  if (disableBlocking) {
    Logger().log("debug", "Request Blocking by Metlo has been disabled")
  }
  METLO_META.disableBlocking = disableBlocking

  startMetlo(
    METLO_META.host,
    METLO_META.key,
    METLO_META.backendPort,
    METLO_META.collectorPort,
    METLO_META.encryptionKey,
    METLO_META.logLevel,
  )
    .then(() => {})
    .catch(err => {
      Logger().log("error", "Metlo encountered an error while initializing ")
      Logger().log("error", err)
    })
}
export default initMiddleware
